/*
canvas d'affichage de la dcomposition d'un entier en facteurs premiers
*/
import java.awt.*;

public class WriteDec extends Canvas {

    public static int[] tabPremiers = 
    {2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,91,97,101};
    public int nbPremiers = 27;
    public int nbMaxi = 10000;
    public int nombre;
    public int[] decomp;
    public int nbDivPrem;
    public boolean erreurNombre=true;
    public Font f1=new Font("Helvetica",Font.PLAIN,10);
    public Font f2=new Font("Helvetica",Font.BOLD,10);

    public WriteDec() {
	super();
        nbPremiers = tabPremiers.length;
	traiteNombre(10);
    }

    public boolean estPremier(int n) {
	boolean rep=true;
	for (int i=0; i<nbPremiers; i++) 
	    if (n % tabPremiers[i] ==0  && n!=tabPremiers[i]) rep=false;
	return rep;
    }

    public int[] decompose(int n) {
	int rep[]=new int[2*nbPremiers+2];
	int rang=0;
	int d;
	int expo;
        for (int i=1; i<=nbPremiers; i++) {
	    d=tabPremiers[i-1];  //diviseur premier
	    expo=0;        //exposant
	    //recherche de l'exposant
	    while (n % d == 0) {
		expo++;
		n=n/d;
	    }	    
	    if (expo!=0) {
		//inscription du diviseur premier et de l'exposant 
		rep[rang]=d;
		rang++;
		rep[rang]=expo;
		rang++;
	    }
	}
	if (n>1) {
	    // n est premier, il faut l'ajouter
	    rep[rang]=n;
	    rang++;
	    rep[rang]=1;
	}
	return rep;
    }

    public void traiteNombre(int n) {
	nombre=n;
	decomp=decompose(nombre);
	nbDivPrem=0;
	int rang=0;
        while (decomp[rang]!=0) {
	    nbDivPrem++;
	    rang=rang+2;
	}
    }


    public void afficheReponse(Graphics g, int x, int y) {
	int expo;
	int rang=0;
	int xpos=x;
        int ypos=y;
        String s=nombre+" = ";
	g.setFont(f1);
	FontMetrics fm=getFontMetrics(getFont());
        g.drawString(s,xpos,ypos);
        xpos=xpos+fm.stringWidth(s);
	while (decomp[rang]!=0)  {
	    s=""+decomp[rang];
	    g.drawString(s,xpos,ypos);
	    xpos=xpos+fm.stringWidth(s);
	    rang++;
	    expo=decomp[rang];
	    rang++;
	    if (expo>1) {
		s=""+expo;
		g.drawString(s,xpos,ypos-8);
	        xpos=xpos+fm.stringWidth(s);
	    }
	    if (decomp[rang]!=0) {
		s=" x ";
		g.drawString(s,xpos,ypos);
	        xpos=xpos+fm.stringWidth(s);
	    }
	} 
        if (estPremier(nombre)) {
	    g.drawString("C'est un nombre premier !",xpos+10,getSize().height-10);
	}
    }
	

    synchronized public void dessineBranches(Graphics g, int niv, int facteur, int x, int y, int hauteur) {
	int dx=(getSize().width-50)/(nbDivPrem+1);
	if (decomp[niv]!=0) {
	    int dh=hauteur/(2*decomp[niv+1]+2);
	    int puiss=1;
	    for (int i=0; i<=decomp[niv+1]; i++) {
		int ny=y-hauteur/2+(2*i+1)*dh;
	        g.setFont(f1);		
                g.drawLine(x,y,x+dx,ny);
		g.drawString(""+puiss,x+dx,ny);
		dessineBranches(g,niv+2,facteur,x+dx,ny,2*dh);
		facteur=facteur*decomp[niv];
		puiss=puiss*decomp[niv];		 
	    }
	}
	else {
	    g.setFont(f2);
	    //g.drawLine(x,y,x+dx/2,y);
	    g.drawString(""+facteur,x+dx/2+10,y);
	}
    }

    public void dessineArbre(Graphics g, int x, int y, int hauteur) {
	g.fillOval(x-3,y-3,6,6);
	dessineBranches(g,0,1,x,y,hauteur);
    }

    public void paint(Graphics g) {
	int y=getSize().height-10;
	if (erreurNombre) {
	    g.drawString("Nombre incorrect !",10,y);
	}
	else if (nombre<=1) {
	    g.drawString("Entrer un entier suprieur ou gal  2 !",10,y);
	}
	else if (nombre>nbMaxi) {
	    g.drawString("Entrer un entier infrieur  10000 !",10,y);
	}
	else {
	    afficheReponse(g,10,y);
	    dessineArbre(g,10,getSize().height/2,getSize().height);
	}
    }

}
